package betterquesting.api2.client.gui.controls;

import betterquesting.api.misc.ICallback;
import betterquesting.api.utils.RenderUtils;
import betterquesting.api2.client.gui.controls.io.FloatSimpleIO;
import betterquesting.api2.client.gui.misc.GuiRectangle;
import betterquesting.api2.client.gui.misc.IGuiRect;
import betterquesting.api2.client.gui.panels.IGuiPanel;
import betterquesting.api2.client.gui.resources.colors.IGuiColor;
import betterquesting.api2.client.gui.resources.textures.IGuiTexture;
import betterquesting.api2.client.gui.themes.presets.PresetColor;
import betterquesting.api2.client.gui.themes.presets.PresetTexture;
import java.util.List;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.FontRenderer;
import net.minecraft.client.gui.GuiScreen;
import net.minecraft.util.ChatAllowedCharacters;
import net.minecraft.util.MathHelper;
import org.lwjgl.input.Keyboard;
import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.GL11;

/* loaded from: betterquesting_api2_client_gui_controls.jar:betterquesting/api2/client/gui/controls/PanelTextField.class */
public class PanelTextField<T> implements IGuiPanel {
    private final IGuiRect transform;
    private IGuiColor colHighlight;
    private IGuiColor colWatermark;
    private String text;
    private IValueIO<Float> scrollX;
    private IValueIO<Float> scrollY;
    private final IFieldFilter<T> filter;
    private ICallback<T> callback;
    private boolean enabled = true;
    private IGuiTexture[] texState = new IGuiTexture[3];
    private IGuiColor[] colStates = new IGuiColor[3];
    private boolean lockFocus = false;
    private boolean isFocused = false;
    private boolean isActive = true;
    private boolean canWrap = false;
    private int maxLength = 32;
    private String watermark = "";
    private int selectStart = 0;
    private int selectEnd = 0;
    private boolean dragging = false;
    private GuiRectangle cursorLine = new GuiRectangle(4, 4, 1, 8);
    private int scrollWidth = 0;
    private int scrollHeight = 0;

    public PanelTextField(IGuiRect rect, String text, IFieldFilter<T> filter) {
        this.transform = rect;
        this.cursorLine.setParent(this.transform);
        this.filter = filter;
        setTextures(PresetTexture.TEXT_BOX_0.getTexture(), PresetTexture.TEXT_BOX_1.getTexture(), PresetTexture.TEXT_BOX_2.getTexture());
        setMainColors(PresetColor.TEXT_AUX_0.getColor(), PresetColor.TEXT_AUX_0.getColor(), PresetColor.TEXT_AUX_0.getColor());
        setAuxColors(PresetColor.TEXT_WATERMARK.getColor(), PresetColor.TEXT_HIGHLIGHT.getColor());
        this.scrollX = new FloatSimpleIO();
        this.scrollY = new FloatSimpleIO();
        setText(text);
    }

    public PanelTextField<T> setCallback(ICallback<T> callback) {
        this.callback = callback;
        return this;
    }

    public PanelTextField<T> setTextures(IGuiTexture disabled, IGuiTexture idle, IGuiTexture focused) {
        this.texState[0] = disabled;
        this.texState[1] = idle;
        this.texState[2] = focused;
        return this;
    }

    public PanelTextField<T> setMainColors(IGuiColor disabled, IGuiColor idle, IGuiColor focused) {
        this.colStates[0] = disabled;
        this.colStates[1] = idle;
        this.colStates[2] = focused;
        return this;
    }

    public PanelTextField<T> setAuxColors(IGuiColor watermark, IGuiColor highlight) {
        this.colWatermark = watermark;
        this.colHighlight = highlight;
        return this;
    }

    public PanelTextField<T> setMaxLength(int size) {
        this.maxLength = size;
        return this;
    }

    public PanelTextField<T> enableWrapping(boolean state) {
        this.canWrap = state;
        updateScrollBounds();
        return this;
    }

    public void lockFocus(boolean state) {
        this.lockFocus = state;
        if (state) {
            this.isFocused = true;
        }
    }

    public PanelTextField<T> setScrollDriverX(IValueIO<Float> driver) {
        this.scrollX = driver;
        return this;
    }

    public PanelTextField<T> setScrollDriverY(IValueIO<Float> driver) {
        this.scrollY = driver;
        return this;
    }

    public int getScrollX() {
        if (this.scrollWidth <= 0) {
            return 0;
        }
        return (int) (this.scrollWidth * this.scrollX.readValue().floatValue());
    }

    public int getScrollY() {
        if (this.scrollHeight <= 0) {
            return 0;
        }
        return (int) (this.scrollHeight * this.scrollY.readValue().floatValue());
    }

    public void setScrollX(int value) {
        if (this.scrollWidth <= 0) {
            this.scrollX.writeValue(Float.valueOf(0.0f));
        } else {
            this.scrollX.writeValue(Float.valueOf(MathHelper.func_76125_a(value, 0, this.scrollWidth) / this.scrollWidth));
        }
    }

    public void setScrollY(int value) {
        if (this.scrollHeight <= 0) {
            this.scrollY.writeValue(Float.valueOf(0.0f));
        } else {
            this.scrollY.writeValue(Float.valueOf(MathHelper.func_76125_a(value, 0, this.scrollHeight) / this.scrollHeight));
        }
    }

    public void setActive(boolean state) {
        this.isActive = state;
    }

    public boolean isActive() {
        return this.isActive;
    }

    public boolean isFocused() {
        return this.isFocused;
    }

    public PanelTextField<T> setWatermark(String text) {
        this.watermark = text;
        return this;
    }

    public void setText(String text) {
        this.text = this.filter.filterText(text);
        updateScrollBounds();
        setCursorPosition(0);
    }

    public String getRawText() {
        return this.text;
    }

    public T getValue() {
        return this.filter.parseValue(getRawText());
    }

    public String getSelectedText() {
        int l = Math.min(this.selectStart, this.selectEnd);
        int r = Math.max(this.selectStart, this.selectEnd);
        return this.text.substring(l, r);
    }

    public void writeText(String in) {
        int used;
        StringBuilder out = new StringBuilder();
        int l = Math.min(this.selectStart, this.selectEnd);
        int r = Math.max(this.selectStart, this.selectEnd);
        int space = (this.maxLength - this.text.length()) - (l - r);
        if (!this.text.isEmpty()) {
            out.append((CharSequence) this.text, 0, l);
        }
        if (space < 0) {
            used = 0;
        } else if (space < in.length()) {
            out.append((CharSequence) in, 0, space);
            used = space;
        } else {
            out.append(in);
            used = in.length();
        }
        if (!this.text.isEmpty() && r < this.text.length()) {
            out.append((CharSequence) this.text, r, this.text.length());
        }
        this.text = this.filter.filterText(out.toString());
        updateScrollBounds();
        moveCursorBy((l - this.selectEnd) + used);
        if (this.callback != null) {
            this.callback.setValue(this.filter.parseValue(this.text));
        }
    }

    public void deleteWords(int num) {
        if (!this.text.isEmpty()) {
            if (this.selectEnd != this.selectStart) {
                writeText("");
            } else {
                deleteFromCursor(getNthWordFromCursor(num) - this.selectStart);
            }
        }
    }

    public void deleteFromCursor(int num) {
        if (!this.text.isEmpty()) {
            if (this.selectEnd != this.selectStart) {
                writeText("");
                return;
            }
            boolean flag = num < 0;
            int i = flag ? this.selectStart + num : this.selectStart;
            int j = flag ? this.selectStart : this.selectStart + num;
            String s = "";
            if (i >= 0) {
                s = this.text.substring(0, i);
            }
            if (j < this.text.length()) {
                s = s + this.text.substring(j);
            }
            this.text = this.filter.filterText(s);
            updateScrollBounds();
            if (flag) {
                moveCursorBy(num);
            }
            if (this.callback != null) {
                this.callback.setValue(this.filter.parseValue(this.text));
            }
        }
    }

    public int getNthWordFromCursor(int numWords) {
        return getNthWordFromPos(numWords, this.selectStart);
    }

    public int getNthWordFromPos(int n, int pos) {
        return getNthWordFromPosWS(n, pos, true);
    }

    public int getNthWordFromPosWS(int n, int pos, boolean skipWs) {
        int i = pos;
        boolean flag = n < 0;
        int j = Math.abs(n);
        for (int k = 0; k < j; k++) {
            if (!flag) {
                int l = this.text.length();
                i = this.text.indexOf(32, i);
                if (i == -1) {
                    i = l;
                } else {
                    while (skipWs && i < l && this.text.charAt(i) == ' ') {
                        i++;
                    }
                }
            } else {
                while (skipWs && i > 0 && this.text.charAt(i - 1) == ' ') {
                    i--;
                }
                while (i > 0 && this.text.charAt(i - 1) != ' ') {
                    i--;
                }
            }
        }
        return i;
    }

    public void moveCursorBy(int num) {
        setCursorPosition(this.selectEnd + num);
    }

    public void setCursorPosition(int pos) {
        this.selectStart = pos;
        int i = this.text.length();
        this.selectStart = MathHelper.func_76125_a(this.selectStart, 0, i);
        setSelectionPos(this.selectStart);
    }

    public boolean onKeyTyped(char typedChar, int keyCode) {
        if (!this.isFocused) {
            return false;
        }
        if (keyCode == 30 && isKeyComboCtrl()) {
            setCursorPosition(this.text.length());
            setSelectionPos(0);
            return true;
        }
        if (keyCode == 46 && isKeyComboCtrl()) {
            GuiScreen.func_146275_d(getSelectedText());
            return true;
        }
        if (keyCode == 47 && isKeyComboCtrl()) {
            if (this.isActive) {
                writeText(GuiScreen.func_146277_j());
                return true;
            }
            return true;
        }
        if (keyCode == 45 && isKeyComboCtrl()) {
            GuiScreen.func_146275_d(getSelectedText());
            if (this.isActive) {
                writeText("");
                return true;
            }
            return true;
        }
        switch (keyCode) {
            case 14:
                if (GuiScreen.func_146271_m()) {
                    if (this.isActive) {
                        deleteWords(-1);
                        return true;
                    }
                    return true;
                }
                if (this.isActive) {
                    deleteFromCursor(-1);
                    return true;
                }
                return true;
            case 28:
                if (this.canWrap) {
                    writeText("\n");
                    return true;
                }
                return true;
            case 199:
                if (GuiScreen.func_146272_n()) {
                    setSelectionPos(0);
                    return true;
                }
                setCursorPosition(0);
                return true;
            case 200:
                return true;
            case 203:
                if (GuiScreen.func_146272_n()) {
                    if (GuiScreen.func_146271_m()) {
                        setSelectionPos(getNthWordFromPos(-1, this.selectEnd));
                        return true;
                    }
                    setSelectionPos(this.selectEnd - 1);
                    return true;
                }
                if (GuiScreen.func_146271_m()) {
                    setCursorPosition(getNthWordFromCursor(-1));
                    return true;
                }
                moveCursorBy(-1);
                return true;
            case 205:
                if (GuiScreen.func_146272_n()) {
                    if (GuiScreen.func_146271_m()) {
                        setSelectionPos(getNthWordFromPos(1, this.selectEnd));
                        return true;
                    }
                    setSelectionPos(this.selectEnd + 1);
                    return true;
                }
                if (GuiScreen.func_146271_m()) {
                    setCursorPosition(getNthWordFromCursor(1));
                    return true;
                }
                moveCursorBy(1);
                return true;
            case 207:
                if (GuiScreen.func_146272_n()) {
                    setSelectionPos(this.text.length());
                    return true;
                }
                setCursorPosition(this.text.length());
                return true;
            case 208:
                return true;
            case 211:
                if (GuiScreen.func_146271_m()) {
                    if (this.isActive) {
                        deleteWords(1);
                        return true;
                    }
                    return true;
                }
                if (this.isActive) {
                    deleteFromCursor(1);
                    return true;
                }
                return true;
            default:
                if (ChatAllowedCharacters.func_71566_a(typedChar)) {
                    if (this.isActive) {
                        writeText(Character.toString(typedChar));
                        return true;
                    }
                    return true;
                }
                return false;
        }
    }

    private static boolean isKeyComboCtrl() {
        return (!GuiScreen.func_146271_m() || GuiScreen.func_146272_n() || Keyboard.isKeyDown(56) || Keyboard.isKeyDown(184)) ? false : true;
    }

    public void setSelectionPos(int position) {
        int i = this.text.length();
        if (position > i) {
            position = i;
        }
        if (position < 0) {
            position = 0;
        }
        if (this.selectEnd != position) {
            this.selectEnd = position;
            FontRenderer font = Minecraft.func_71410_x().field_71466_p;
            if (this.canWrap) {
                List<String> lines = RenderUtils.splitStringWithoutFormat(this.text, getTransform().getWidth() - 8, font);
                String lastFormat = "";
                int idx = 0;
                int y = 0;
                int x = 0;
                while (true) {
                    if (y >= lines.size()) {
                        break;
                    }
                    String s = lines.get(y);
                    if (this.selectEnd >= idx) {
                        if (this.selectEnd < idx + s.length() + (y == lines.size() - 1 ? 1 : 0)) {
                            x = RenderUtils.getStringWidth(lastFormat + s.substring(0, this.selectEnd - idx), font);
                            break;
                        }
                    }
                    idx += s.length();
                    lastFormat = RenderUtils.getFormatFromString(lastFormat + s);
                    y++;
                }
                int y2 = y * font.field_78288_b;
                int sy = getScrollY();
                if (y2 < sy) {
                    setScrollY(y2);
                } else if (y2 > (sy + (this.transform.getHeight() - 8)) - font.field_78288_b) {
                    setScrollY((y2 - (this.transform.getHeight() - 8)) + font.field_78288_b);
                }
                this.cursorLine.x = x + 4;
                this.cursorLine.y = y2 + 4;
                this.cursorLine.w = 1;
                this.cursorLine.h = font.field_78288_b;
                return;
            }
            int x2 = RenderUtils.getStringWidth(this.text.substring(0, this.selectEnd), font);
            int sx = getScrollX();
            if (x2 < sx) {
                setScrollX(x2);
            } else if (x2 > sx + (this.transform.getWidth() - 8)) {
                setScrollX(x2 - (this.transform.getWidth() - 8));
            }
            this.cursorLine.x = x2 + 4;
            this.cursorLine.y = 4;
            this.cursorLine.w = 1;
            this.cursorLine.h = font.field_78288_b;
        }
    }

    public void updateScrollBounds() {
        FontRenderer font = Minecraft.func_71410_x().field_71466_p;
        int prevX = getScrollX();
        int prevY = getScrollY();
        if (!this.canWrap) {
            this.scrollHeight = 0;
            this.scrollWidth = Math.max(0, RenderUtils.getStringWidth(this.text, font) - (this.transform.getWidth() - 8));
        } else {
            this.scrollWidth = 0;
            this.scrollHeight = Math.max(0, (RenderUtils.splitString(this.text, this.transform.getWidth() - 8, font).size() * font.field_78288_b) - (this.transform.getHeight() - 8));
        }
        setScrollX(prevX);
        setScrollY(prevY);
    }

    public IGuiRect getTransform() {
        return this.transform;
    }

    public void initPanel() {
    }

    public void setEnabled(boolean state) {
        this.enabled = state;
    }

    public boolean isEnabled() {
        return this.enabled;
    }

    public void drawPanel(int mx, int my, float partialTick) {
        Minecraft mc = Minecraft.func_71410_x();
        if (this.isActive && this.dragging && Mouse.isButtonDown(0)) {
            if (this.canWrap) {
                setSelectionPos(RenderUtils.getCursorPos(this.text, (mx - (this.transform.getX() + 4)) + getScrollX(), (my - (this.transform.getY() + 4)) + getScrollY(), this.transform.getWidth() - 8, mc.field_71466_p));
            } else {
                setSelectionPos(RenderUtils.getCursorPos(this.text, (mx - (this.transform.getX() + 4)) + getScrollX(), mc.field_71466_p));
            }
        } else if (this.dragging) {
            this.dragging = false;
        }
        IGuiRect bounds = getTransform();
        int state = !isActive() ? 0 : this.isFocused ? 2 : 1;
        IGuiTexture t = this.texState[state];
        GL11.glPushMatrix();
        if (t != null) {
            t.drawTexture(bounds.getX(), bounds.getY(), bounds.getWidth(), bounds.getHeight(), 0.0f, partialTick);
        }
        RenderUtils.startScissor(bounds);
        GL11.glTranslatef(-getScrollX(), -getScrollY(), 0.0f);
        if (this.text.length() <= 0) {
            if (!this.isFocused) {
                mc.field_71466_p.func_85187_a(this.watermark, bounds.getX() + 4, bounds.getY() + 4, this.colWatermark.getRGB(), false);
            }
        } else {
            IGuiColor c = this.colStates[state];
            if (!this.canWrap) {
                RenderUtils.drawHighlightedString(mc.field_71466_p, this.text, bounds.getX() + 4, bounds.getY() + 4, c.getRGB(), false, this.colHighlight.getRGB(), this.selectStart, this.selectEnd);
            } else {
                RenderUtils.drawHighlightedSplitString(mc.field_71466_p, this.text, bounds.getX() + 4, bounds.getY() + 4, bounds.getWidth() - 8, c.getRGB(), false, this.colHighlight.getRGB(), this.selectStart, this.selectEnd);
            }
        }
        if (this.isFocused && this.selectStart == this.selectEnd && (System.currentTimeMillis() / 500) % 2 == 0) {
            RenderUtils.drawHighlightBox(this.cursorLine, this.colHighlight);
        }
        RenderUtils.endScissor();
        GL11.glPopMatrix();
    }

    public boolean onMouseClick(int mx, int my, int button) {
        if (this.transform.contains(mx, my)) {
            if (!this.isFocused) {
                this.isFocused = true;
                updateScrollBounds();
            }
            if (this.canWrap) {
                setCursorPosition(RenderUtils.getCursorPos(this.text, (mx - (this.transform.getX() + 4)) + getScrollX(), (my - (this.transform.getY() + 4)) + getScrollY(), this.transform.getWidth() - 8, Minecraft.func_71410_x().field_71466_p));
            } else {
                setCursorPosition(RenderUtils.getCursorPos(this.text, (mx - (this.transform.getX() + 4)) + getScrollX(), Minecraft.func_71410_x().field_71466_p));
            }
            this.dragging = true;
            return false;
        }
        if (this.isFocused && !this.lockFocus) {
            this.isFocused = false;
            this.text = this.filter.parseValue(this.text).toString();
            return false;
        }
        return false;
    }

    public boolean onMouseRelease(int mx, int my, int button) {
        return this.isFocused && this.dragging;
    }

    public boolean onMouseScroll(int mx, int my, int scroll) {
        if (this.isFocused && this.transform.contains(mx, my) && this.canWrap) {
            setScrollY(getScrollY() + (scroll * 4));
            return true;
        }
        return false;
    }

    public List<String> getTooltip(int mx, int my) {
        return null;
    }
}
